/* libzzsock
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2000-2006, Toni Thomsson <toni@tonjac.org> 
*/

#ifndef zz_socket_h
#define zz_socket_h

#ifdef _WIN32
	#include <winsock2.h>
#else
	#include <sys/types.h>
	#include <sys/socket.h>	
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <netdb.h>
	#include <unistd.h>
	#include <errno.h>
    /** Handle */
	typedef int SOCKET;
	#define INVALID_SOCKET -1
	#define SOCKET_ERROR -1
#endif

#include <zz_string.h>
#include <zz_exception.h>
#include <zz_sock_exception.h>
#include <zz_export.h>

/**
    TCP socket

    @author Toni Thomsson, toni@tonjac.org
*/
class ZZ_API CzzSocket
{
public:

	/**
        Create a listening server socket
        @param  port    Port
        @param  que     Number of callers that can be queued, use SOMAXCONN for max.
	*/
	CzzSocket( const char* port,
				int que	);

	/**
	    Connect as a client
        @param  host    Hostname
        @param  port    Port
        @param  s_timeout   Send timeout i s
        @param  r_timeout   Receive timeout i s

	*/
	CzzSocket( const char* host,
				const char* port,
				unsigned short s_timeout,
				unsigned short r_timeout );

	/**
		Create a socket from an existing handle
        @param  socket  Handle
        @param  host    Hostname

	*/
	CzzSocket( SOCKET socket,
				const char* host );

	/**
		Copy construktor
	*/
	CzzSocket( CzzSocket& obj );
	
    /**
		Create a socket thats not connected. Use ConnectAsClient or ConnectAsServer
        to connect.
	*/
	CzzSocket( void );
	
	virtual ~CzzSocket();

	/**
        Accept call. Blocks until a call is done.
	*/
	CzzSocket* Accept( void );

	/**
        Read from socket
        @param  receiveBuffer   Buffer for read data
        @param  bufferSize      Size of buffer
        @param  bytesAquired    Number of bytes to read
        @param  flags           Flags, see recv()
	*/
	int Receive( char* receiveBuffer,
					int bufferSize,
					int bytesAquired,
					int flags );

	/**
		Send data
        @param  sendBuffer      Buffer to send
        @param  bufferSize      Size of buffer
        @param  flags           Flags, see send()
	*/
	int Send( char* sendBuffer,
				int bufferSize,
				int flags );

	/**
        Connect as a server
	*/
	void ConnectAsServer( void );

	/**
        Connect as a client
	*/
	void ConnectAsClient( void );
	
	/**
        Disconnect
	*/
	void Disconnect( void );

	/** Handle */
	SOCKET GetSocket( void ) { return m_Socket; };
	
	/** Host */
	CzzString GetHost( void ) { return m_Host; };

	/** Port */
	CzzString GetPort( void ) { return m_Port; };

	/** Queue */
	unsigned int GetQue( void ) { return m_Que; };

	/** Send timeout */
	unsigned short GetSendTimeout( void ) { return m_SendTimeout; };

	/** Receive timeout */
	unsigned short GetReceiveTimeout( void ) { return m_ReceiveTimeout; };

	/** Hostname */
	void SetHost( const CzzString& host ) { m_Host = host; };

	/** Port */
	void SetPort( const CzzString& port ) { m_Port = port; };

	/** Queue */
	void SetQue( unsigned int que ) { m_Que = que; };

	/** Send timeout */
	void SetSendTimeout( unsigned short to ) { m_SendTimeout = to; };

	/** Receive timeout */
	void SetReceiveTimeout( unsigned short to ) { m_ReceiveTimeout = to; };

	/** Current host */
	CzzString GetCurrentHost( void );

    /** Non blocking */
	void setNonBlocking( bool nonblock );
	
	/**
        Init sockets. Should only be called one time.
	*/
	static void Init( void );

	/**
        Exit sockets. Should only be called one time.
	*/
	static void Exit( void );

protected:

    /** Handle */
	SOCKET m_Socket;
	
    /** Host */
    CzzString m_Host;
	
    /** Port */
    CzzString m_Port;
	
    /** Queue */
    unsigned int m_Que;
	
    /** Send timeout */
    unsigned short m_SendTimeout;

    /** Receive timeout */
	unsigned short m_ReceiveTimeout;

    /** Data to read? */
	bool isDataAvailable( SOCKET sock );
};

#endif // zz_socket_h


